import {
  ProseEditor,
  ProseEditorOverlayTools,
  ScrollPane,
  SplitPane,
  TOCProvider
} from 'substance'

import Comments from './panes/Comments/CommentBoxList'
import CommentsProvider from './panes/Comments/CommentsProvider'
import ContainerEditor from './ContainerEditor'
// import Overlay from './Overlay'
import Notes from './panes/Notes/Notes'
import NotesProvider from './panes/Notes/NotesProvider'
import TableOfContents from './panes/TableOfContents/TableOfContents'

class Editor extends ProseEditor {
  constructor (parent, props) {
    super(parent, props)

    this.handleActions({
      'showComments': function () { this.toggleCommentsArea(true) },
      'hideComments': function () { this.toggleCommentsArea(false) }
    })
  }

  willUpdateState () {}

  didMount () {
    this.documentSession.on('didUpdate', this.documentSessionUpdated, this)
    this.extendState({ editorReady: true })
  }

  render ($$) {
    var el = $$('div').addClass('sc-prose-editor')

    // left side: editor and toolbar
    var toolbar = this._renderToolbar($$)
    var editor = this._renderEditor($$)

    var footerNotes = $$(Notes)
    var props = {
      book: this.props.book,
      fragment: this.props.fragment,
      history: this.props.history
    }

    var toc = $$(TableOfContents, props)

    var editorWithFooter = $$('div')
      .append(
        editor,
        footerNotes
      )

    var commentsPane = this.state.editorReady
    ? $$(Comments, {
      // TODO -- should probably remove the || {}
      comments: this.props.fragment.comments || {},
      update: this.props.updateComments,
      user: this.props.user
    }).addClass('sc-comments-pane')
    : $$('div')

    var editorWithComments = $$(SplitPane, { sizeA: '100%', splitType: 'vertical' })
      .append(
        editorWithFooter,
        commentsPane
      )

    var contentPanel = $$(ScrollPane, {
      scrollbarPosition: 'right',
      overlay: ProseEditorOverlayTools
    })
    .append(editorWithComments)
    .attr('id', 'content-panel')
    .ref('contentPanel')

    var contentPanelWithSplitPane = $$(SplitPane, { sizeA: '75%', splitType: 'vertical' })
      .append(
        contentPanel,
        toc
      )

    el.append(
      $$(SplitPane, { splitType: 'horizontal' })
        .append(
          toolbar,
          contentPanelWithSplitPane
        )
    )

    return el
  }

  _renderEditor ($$) {
    const configurator = this.props.configurator
    const editing = this.props.disabled ? 'selection' : 'full'
    return $$(ContainerEditor, {
      editing: editing,
      documentSession: this.documentSession,
      commands: configurator.getSurfaceCommandNames(),
      containerId: 'body',
      textTypes: configurator.getTextTypes(),
      trackChanges: this.props.trackChanges
    }).ref('body')
  }

  getInitialState () {
    return {
      editorReady: false
    }
  }

  scrollTo (nodeId) {
    this.refs.contentPanel.scrollTo(nodeId)
  }

  toggleCommentsArea (show) {
    var self = this

    setTimeout(function () {
      var el = self.refs.contentPanel

      if (show && !el.hasClass('sc-has-comments')) {
        el.addClass('sc-has-comments')
      }

      if (!show && el.hasClass('sc-has-comments')) {
        el.removeClass('sc-has-comments')
      }
    })
  }

  getChildContext () {
    const oldContext = super.getChildContext()
    const doc = this.doc

    // toc provider
    const tocProvider = new TOCProvider(doc, {
      containerId: 'body'
    })

    // // notes provider
    const notesProvider = new NotesProvider(doc)

    // // comments provider
    const commentsProvider = new CommentsProvider(doc, {
      commandManager: this.commandManager,
      comments: this.props.fragment.comments,
      containerId: this.props.containerId,
      documentSession: this.documentSession,
      surfaceManager: this.surfaceManager,
      toggleCommentsArea: this.toggleCommentsArea,
      updateComments: this.props.updateComments
    })

    // attach all to context
    return {
      ...oldContext,
      tocProvider,
      notesProvider,
      commentsProvider
    }
  }
}

export default Editor
